<?php

class Controller_Api_V0_StudyPlan extends Controller_Handler_Api
{
    private function normalizeDisciplinesData($data, $facultyID) {
        if (!isset($data->subjectID) && isset($data->externalID)) {
            $data->subjectID = Model_Subject::withExternalID($data->id, $data->name, '', $facultyID);
        }

        foreach ($data->teachers as $teacher) {
            $ID = Model_Teacher::withINILA($teacher->hashSnils);
            if ($ID != -1)
                $data->teacherIDs[] = $ID;
        }
        $data->teacherIDs[] = 1;

        /*
         * Если нет ни одного преподавателя, то ругаемся в логи и пропускаем дисциплину до лучших времен
         */
        if (!isset($data->teacherIDs) || count($data->teacherIDs) == 0) {
            Log::instance()->add(Log::WARNING, "A discipline without teachers is arrived: {0}",
                array(
                    '{0}' => print_r($data, TRUE)
                )
            );
            return null;
        }

        switch ($data->type) {
            case 'Экзамен':
                $data->type = Model_Discipline::EXAM;
                break;
            case 'Зачет':
                $data->type = Model_Discipline::CREDIT;
                break;
            default:
                Log::instance()->add(Log::WARNING, "A discipline has incorrect type: {0}",
                    array(
                        '{0}' => print_r($data, TRUE)
                    )
                );
                return null;
        }

        return $data;
    }

    private function processStudyPlan($data, $year, $facultyID) {
        if (!isset($data->id) && isset($data->externalID)) {
            $ID = Model_Plan::withExternalID($data->externalID);
            if ($ID == -1) {
                $data->planModel = Model_Plan::make()
                    ->externalID($data->externalID)
                    ->year($year)
                    ->faculty($facultyID)
                    ->create();
            }
            else {
                $data->planModel = Model_Plan::load($ID);
            }
        }

        foreach ($data->disciplines as &$discipline) {
            $discipline = $this->normalizeDisciplinesData($discipline, $facultyID);
        }
        return $data;
    }

    private function processDisciplines($data, $semesterID, $facultyID) {
        $res = [];
        try {
//            echo Debug::vars($data->disciplines);
            foreach ($data->disciplines as $discipline) {
                if ($discipline == null)
                    continue;

                if (isset($discipline->id)) {
                    // ToDo: update discipline
                } else {
                    print_r($discipline);

                    $plan = Model_Plan::load($data->id);
                    $gradeID = $plan->GradeID;
                    $newDiscipline = Model_Discipline::make()
                        ->author($discipline->teacherIDs[0])
                        ->subject($discipline->SubjectID)
                        ->semester($semesterID)
                        ->type($discipline->type)
                        ->faculty($facultyID)
                        ->grade($gradeID)
                        ->create();
                    $res[] = $newDiscipline->ID;
                }
            }
        } catch (InvalidArgumentException $e) {
            $this->badRequestError($this->makeErrorMsg($data, $e->getMessage()));
        }
        return $res;
    }

    private function processData($data, $requestInfo) {
        $data = $this->processStudyPlan($data, $requestInfo['Year'], $requestInfo['FacultyID']);
        return $this->processDisciplines($data, $requestInfo['SemesterID'], $requestInfo['FacultyID']);
    }

    /**
     * @api {put} api/v0/studyPlan Add new study plan(s)
     * @apiName Add new study plan
     * @apiGroup Study plans
     * @apiVersion 0.1.2
     * @apiParam {String} token Api key
     * @apiParam {String} [batch] Flag about massive addition
     * @apiParam {String} externalId
     * @apiParam {Array} disciplines
     */
    public function action_put_index() {
        try {
            if ($this->request->query('batch') !== NULL) {
                $data = json_decode($this->request->body());

                $data->year = (int)$data->year;
                $data->semester = (int)$data->semester;

                $requestInfo = array(
                    'Year' => $data->year,
                    'SemesterID' => Model_Semesters::find($data->year, $data->semester),
                    'FacultyID' => Model_Faculties::getIdByExternalID($data->faculty),
                );

                $res = [];
                foreach ($data->plans as $item) {
                    $res[] = $this->processData($item, $requestInfo);
                    if (count($res) > 10) {
                        echo Debug::vars($res); exit;
                    }
//                    return $res;
                }

            } else {
                throw new InvalidArgumentException('No batch data received!');
//                $data = $this->request->query();
//                $res[] = $this->processData($data);
            }
        } catch (Exception $e) {
            $this->badRequestError($e->getMessage());
        }

        return $res;
    }
}
