<?php defined('SYSPATH') or die('No direct script access.');

class FileParser
{
    /**
     * Parse students' info & synchronize it with database.
     * @param $filename string File with source data
     * @param $facultyID int
     * @return array errors
     */
    public static function uploadStudents($filename, $facultyID) {
        $content = explode("\r\n", self::toUTF8(file_get_contents($filename)));

        $i = 0;
        $errors = [];
        foreach ($content as $line) {
            $line = explode(';', $line);
            $line = Arr::map('trim', $line);

            // ФИО, курс, степень подготовки,
            list($name, $gradeNum, $groupNum, $degree, $spec) = UTF8::clear($line);

            // Проверим, что входная строка не пуста.
            $name = trim($name);
            if (empty($name))
                continue;

            // Фамилия, имя, отчество
            list($lastName, $firstName, $secondName) = Text::parseFullName($name);

            $degree = self::getDegreeType($degree);

            $attempt = Model_Student::createStudentEx(
                $lastName, $firstName, $secondName, $gradeNum, $groupNum, $degree, $spec, $facultyID
            );

            if ($attempt == -1) {
                $errors[] = ['Row' => $i, 'Info' => implode(';', $line)];
            }

            $i++;
        }
        return $errors;
    }

    private static function getDegreeType($degree) {
        switch ($degree) {
            case 'Бакалавр':
                return 'bachelor';
            case 'Магистр':
                return 'master';
            case 'Специалист':
                return 'specialist';
            default:
                return $degree;
        }
    }

    /**
     * Parse subjects' info & synchronize it with database.
     * @param $filename  string File with source data
     * @param $facultyID int
     * @return array
     */
    public static function uploadSubjects($filename, $facultyID) {
        if (File::mime($filename) != 'text/plain')
            return [];

        $file = file_get_contents($filename);
        $content = explode("\r\n", self::toUTF8($file)); // DO NOT REPLACE " BY ' THERE!

        $count = $exists = 0;
        $errors = [];

        foreach ($content as $line) {
            $line = explode(';', $line);
            $line = Arr::map('trim', $line);

            list($name, $abbr) = UTF8::clear($line);

            // Проверим, что входная строка не пуста.
            $name = trim($name);
            if (empty($name))
                continue;

            $attempt = Model_Subject::createOrUpdate($name, $abbr, $facultyID);

            if ($attempt < 0) {
                $errors[] = ['Row' => $count, 'Info' => implode(';', $line)];
            } elseif ($attempt == 1) {
                $exists++;
            }

            $count++;
        }

        return [
            'Errors'             => $errors,
            'ErrorsCount'        => count($errors),
            'RecordsCount'       => $count,
            'RecordsExistsCount' => $exists,
        ];
    }

    public static function toUTF8($source_str) {
        // TODO: make it able to convert from any encoding to UTF-8
        $enc = mb_detect_encoding($source_str, mb_list_encodings(), true);
        if ($enc === "ISO-8859-1" || !$enc)
            return mb_convert_encoding($source_str, "utf-8", "windows-1251");
        else if ($enc !== "UTF-8")
            return mb_convert_encoding($source_str, "utf-8", $enc);
        else
            return $source_str;
    }
}
