<?php defined('SYSPATH') or die('No direct script access.');

class Model_Helper_DisciplineBuilder extends Model_Helper_Builder
{
    public function create() {
        $this->data += [
            'Subtype' => null,
            'IsLocked' => false,
            'Milestone' => 0,
            'Lectures' => 0,
            'Practice' => 0,
            'Labs' => 0,
            'GradeID' => null
        ];

        $required = [
            'Subtype', 'IsLocked', 'Milestone', 'AuthorID', 'GradeID', 'FacultyID',
            'SubjectID', 'Lectures', 'Practice', 'Labs', 'Type', 'SemesterID'
        ];

        if (array_diff($required, array_keys($this->data)))
            throw new InvalidArgumentException('not enough arguments');

        return new Model_Discipline($this->data, false);
    }

    // todo: think about kohana validator

    function & author($id) {
        if (!is_numeric($id) || $id <= 0)
            throw new InvalidArgumentException('author id is incorrect');
        $this->data['AuthorID'] = (int) $id;
        return $this;
    }

    function & grade($id) {
        if (!is_numeric($id) || $id <= 0)
            throw new InvalidArgumentException('grade id is incorrect');
        $this->data['GradeID'] = (int) $id;
        return $this;
    }

    function & faculty($id) {
        if (!is_numeric($id) || $id <= 0)
            throw new InvalidArgumentException('faculty id is incorrect');
        $this->data['FacultyID'] = (int) $id;
        return $this;
    }

    function & subject($id) {
        if (!is_numeric($id) || $id <= 0)
            throw new InvalidArgumentException('subject id is incorrect');
        $this->data['SubjectID'] = (int) $id;
        return $this;
    }

    function & lectures($hours) {
        if (!is_numeric($hours) || $hours < 0)
            throw new InvalidArgumentException('lectures hours are incorrect');
        $this->data['Lectures'] = (int) $hours;
        return $this;
    }

    function & practice($hours) {
        if (!is_numeric($hours) || $hours < 0)
            throw new InvalidArgumentException('practice hours are incorrect');
        $this->data['Practice'] = (int) $hours;
        return $this;
    }

    function & labs($hours) {
        if (!is_numeric($hours) || $hours < 0)
            throw new InvalidArgumentException('labs hours are incorrect');
        $this->data['Labs'] = (int) $hours;
        return $this;
    }

    function & type($name) {
        if (!is_string($name))
            throw new InvalidArgumentException('Type is incorrect');

        switch ($name) {
            case Model_Discipline::EXAM:
            case Model_Discipline::CREDIT:
            case Model_Discipline::GRADING_CREDIT:
                break;
            default:
                throw new InvalidArgumentException('Type is incorrect');
        }

        $this->data['Type'] = $name;
        return $this;
    }

    function & semester($id) {
        if (!is_numeric($id) || $id <= 0)
            throw new InvalidArgumentException('semester id is incorrect');
        $this->data['SemesterID'] = (int) $id;
        return $this;
    }
}